function [RadOut,RStatsOut]=CleanHFRadialsFirstOrder(Rorig,RadialStats,CleaningParameters)
%
%  [RadOut,RStatsOut]=CleanHFRadialsFirstOrder(Rorig,RadialStats,CleaningParameters);
%
%  This function cleans the radials (removes those radials that do not meet
%  the criteria set by the user).  All radials from a specified time are
%  simply removed, not just set to NaN.  RadOut is therefore either equal
%  to Rorig if no radials (times) at a specific site are removed, or a
%  subset.
%
%  RStatsOut is a subset of RadialStats
%
%  Note:  Empty structures are stripped off in this at the end, just in
%  case the user does not set those flags (DataFound, etc.).  Will save space in the long
%  run.
%
%  Rorig are the loaded radials.  Not separated by site or gridded.
%  RadialStats is the radial stats structure.  One entry per Rorig entry.
%  CleaningParameters is the cell array of parameters to use when cleaning.
%      Can specify different paramters for different sites.  For example:
%      CleaningParameters={'BMLR',[90 180],...
%         'MaxBear','RangeGood';'MEND',[70],'PerDiffFwd','BelowBad'};
%
%  Format of Cleaning Parameters is:
%     {'SiteCode',[parameters],'StatisticName','UseOfStatistic'; etc.}
%   size of cell array CleaningParameters will be nx4, where n is the
%   number of flags set
%
%   where:  Site Code is the appropriate site code.  See Rorig and
%   RadialStats structures for the site codes.  Case (upper/lower) matters!
%           parameters can be a single parameter or a 2 element parameter
%           if a range will be used
%           StatisticName = the field to use for cleaning, from RadialStats
%           UseOfStatistic - how to flag.  Valid Entries are:
%               BelowBad - radials with statistic below (not below and equal to)
%                   parameter are bad
%               AboveBad - radials with statistic above parameter are bad
%               RangeGood - radials with statistic in range are good
%               RangeBad - radials statistic in range are bad
%
%  If a given field is not found in the RadialStats structure, a warning is
%  printed to the screen.  Processing does not stop, the field is simply
%  skipped.
%
%  Step 1 - set up "goodness array".  This will be an array of ones.  The
%  ones will be set to zeros as radial sets are found that do not meet the
%  specified criteria.
%
%  CMH OCT 2007
%
ArrayToKeep=ones(size(Rorig));
[SiteNameArray{1:numel(Rorig)}] = deal(Rorig.SiteName);
%
[NumFlagsSet,NumCols]=size(CleaningParameters);
%
for jflag=1:NumFlagsSet,
    if isfield(RadialStats,CleaningParameters{jflag,3}),
        eval(['TemSer=RadialStats.' CleaningParameters{jflag,3} ';'])
        switch upper(CleaningParameters{jflag,4})
            case 'BELOWBAD'
                FlagVal=CleaningParameters{jflag,2};
                FlagInd=find(TemSer<FlagVal);
            case 'ABOVEBAD'
                FlagVal=CleaningParameters{jflag,2};
                FlagInd=find(TemSer>FlagVal);
            case 'RANGEGOOD'
                FlagVal=CleaningParameters{jflag,2};
                FlagInd=find((TemSer<FlagVal(1))|(TemSer>FlagVal(2)));
            case 'RANGEBAD'
                FlagVal=CleaningParameters{jflag,2};
                FlagInd=find((TemSer>=FlagVal(1))&(TemSer<=FlagVal(2)));
        end
        TemmarkSite=find(ismember(SiteNameArray,CleaningParameters{jflag,1}));
        %
        %  the indices of the radials to flag will be the intersection
        %  of the FlagVal and the TemmarkSite indices
        %
        FlagIndices=intersect(FlagInd,TemmarkSite);
        %
        if numel(FlagIndices)>0,
            ArrayToKeep(FlagIndices)=0;
        end
        %
    else
        %
        TemStr=['The field ' CleaningParameters{jflag,3} ' Not Found'];
        disp(TemStr)
        TemStr='Cleaning Continuing';
        disp(TemStr)
        TemStr='   ';
        disp(TemStr)
    end
end
%
TemToKeep=find(ArrayToKeep>0);
if numel(TemToKeep)>0,
    RadOut=Rorig(TemToKeep);
    RStatsOut=ExtractFieldsFromStruct(RadialStats,TemToKeep,1);
else
    RadOut=[];
    RStatsOut=[];
end
%
%  now strip off empties
%
if numel(RadOut)>0,
    clear SiteNameArray
    [SiteNameArray{1:numel(RadOut)}] = deal(RadOut.SiteName);
    ValidSiteNums=find(~ismember(SiteNameArray,''));
    ValidSiteNums=ValidSiteNums';
    if numel(ValidSiteNums)>0,
        RadOut=RadOut(ValidSiteNums);
        RStatsOut=ExtractFieldsFromStruct(RStatsOut,ValidSiteNums,1);
    else
        RadOut=[];
        RStatsOut=[];
    end
end
%
%  the routine is done
%
return;
%
        