function [TUVOut]=CombineHFRadarTotals(TUV1,TUV2)
%
%  [TUVOut]=CombineHFRadarTotals(TUV1,TUV2);
%
%  Takes two totals structures, ouutputs a new one which is an average of
%  the input.
%
%  Method:  (1) Gets a new grid which is a combination of both input grids
%           (2)  Outputs the average of the currents at each new gridpoint
%                 and timestamp.
%
%  No corrections for different grid resolutions, etc., are done.  Grids in
%  common result in averages of the currents, grids not in common result
%  in currents that are simply carrried into the result from the respective
%  structure that the gridpoint came from.
%
%
%  CMH Aug 2008
%
%  Step 1a - find grids in common
%
[C,CommonIndices,IB]=intersect(TUV1.LonLat,TUV2.LonLat,'rows');
if numel(CommonIndices)>0,  % saves alot of if statements later
    NewGridMiddlePart=TUV1.LonLat(CommonIndices,:);
else
    NewGridMiddlePart=[NaN NaN];
end
%
%  Step 1b - find grids in Grid 1 that are not in Grid 2
%  and vice versa
%
[C,UniqueGrid1,UniqueGrid2]=setxor(TUV1.LonLat,TUV2.LonLat,'rows');
if numel(UniqueGrid1)>0,  % saves alot of if statements later
    NewGridTopPart=TUV1.LonLat(UniqueGrid1,:);
else
    NewGridTopPart=[NaN NaN];
end
if numel(UniqueGrid2)>0,  % saves alot of if statements later
    NewGridBottomPart=TUV2.LonLat(UniqueGrid2,:);
else
    NewGridBottomPart=[NaN NaN];
end
%
%  Step 1c - Make a new overall grid
%
NewGrid=[NewGridTopPart;NewGridMiddlePart;NewGridBottomPart];
temmark=find(~isnan(NewGrid(:,1)));
NewGrid=NewGrid(temmark,:);
%
% Step 2a - get timestamps
%
TemVec=datevec(TUV1.TimeStamp);
%  deal with datevec roundoff
temmark=find((TemVec(:,5)>58)&(TemVec(:,6)>58));
if numel(temmark)>0,
    TemVec(temmark,4)=TemVec(temmark,4)+1;
    TemVec(temmark,5)=0;
    TemVec(temmark,6)=0;
end
%
DateVector1=round(TemVec);
%
TemVec=datevec(TUV2.TimeStamp);
%  deal with datevec roundoff
temmark=find((TemVec(:,5)>58)&(TemVec(:,6)>58));
if numel(temmark)>0,
    TemVec(temmark,4)=TemVec(temmark,4)+1;
    TemVec(temmark,5)=0;
    TemVec(temmark,6)=0;
end
%
DateVector2=round(TemVec);
%
NewDateVector=unique([DateVector1;DateVector2],'rows');
NewTimeStamp=datenum(NewDateVector);
%  in case NewDateVector isn't sorted
[NewTimeStamp,IndexTime]=sort(NewTimeStamp,'ascend');
NewDateVector=NewDateVector(IndexTime,:);
NewTimeStamp=NewTimeStamp';
%
%  Step 3 - Set up the structure
%
[NumGridPoints]=numel(NewGrid(:,1));
[NumTimes]=numel(NewTimeStamp);
TUVOut=TUVstruct([NumGridPoints NumTimes]);
%
TUVOut.DomainName=['Combined: ' TUV1.DomainName TUV2.DomainName];
TUVOut.CreationInfo=['Combined: ' TUV1.CreationInfo TUV2.CreationInfo];
%
TUVOut.LonLat=NewGrid;
TUVOut.TimeStamp=NewTimeStamp;
%
%  Step 4 - Find the coords of the new grids that the old grids map into
%    No checks for existence here b/c this has to work - the old grid is a
%    combo of the other two
%
[C,NewGrids1,OldGrids1]=intersect(NewGrid,TUV1.LonLat,'rows');
[C,NewGrids2,OldGrids2]=intersect(NewGrid,TUV2.LonLat,'rows');
%
%  Step 5 - now loop through by time and remap / combine
%
for jtime=1:numel(NewTimeStamp),
    %
    [C,NewTime1,OldTime1]=intersect(NewDateVector(jtime,:),DateVector1,'rows');
    [C,NewTime2,OldTime2]=intersect(NewDateVector(jtime,:),DateVector2,'rows');
    %
    if numel(OldTime1)>0,
        %
        OldMeasU1=NaN*ones(numel(NewGrid(:,1)),1);
        OldMeasU1(NewGrids1)=TUV1.U(OldGrids1,OldTime1);
        %
        OldMeasV1=NaN*ones(numel(NewGrid(:,1)),1);
        OldMeasV1(NewGrids1)=TUV1.V(OldGrids1,OldTime1);
        %
    else
        %
        OldMeasU1=NaN*ones(numel(NewGrid(:,1)),1);
        OldMeasV1=NaN*ones(numel(NewGrid(:,1)),1);
        %
    end
    %
    if numel(OldTime2)>0,
        %
        OldMeasU2=NaN*ones(numel(NewGrid(:,1)),1);
        OldMeasU2(NewGrids2)=TUV2.U(OldGrids2,OldTime2);
        %
        OldMeasV2=NaN*ones(numel(NewGrid(:,1)),1);
        OldMeasV2(NewGrids2)=TUV2.V(OldGrids2,OldTime2);
        %
    else
        %
        OldMeasU2=NaN*ones(numel(NewGrid(:,1)),1);
        OldMeasV2=NaN*ones(numel(NewGrid(:,1)),1);
        %
    end
    %
    TemU=nanmean([OldMeasU1';OldMeasU2']);
    TUVOut.U(:,jtime)=TemU';
    TemV=nanmean([OldMeasV1';OldMeasV2']);
    TUVOut.V(:,jtime)=TemV';
    %
end
%
%  the routine is done
%
return;
%
