function [TUVOut]=GetHFTemporalStatsTest(TUVIn,MaxGapTDiff,PerCovWindowSearch)
%
%  [TUVOut]=GetHFTemporalStatsTest(TUVIn,MaxGapTDiff,PerCovWindowSearch);
%
%  CMH - redone to remove time stamps that are NaN rather than
%  interpolating to get them.  12 June 2009.  This could lead to some
%  trouble if teh data is *really gappy* and environmental conditions /
%  currents change alot.
%
%  CMH, Sept 2007
%
%  Percent Coverage Window Calculations Redone Nov 2008
%
%  Inputs:  TUVIn - Either a Tuv structure or a radial structure for a
%           single site (as found in Rcat).  Each site must be passed singly to this routine
%           if a radial structure is passed.  It is assumed that the passed
%           structure is organized from earliest time to latest time.
%
%           MaxGapTDiff - the maximum gap allowed when getting the time
%           difference, in hours (b/c the input structure can have gaps)
%           This is a one sided difference, so MaxGapTDiff of 3 meanss that
%           the forward derivative and teh backward derivatiive will be
%           caluclated separately with gaps of up to 3 hours
%
%           PerCovWindowSearch - the percentage of time that there are
%           measurements within the specified window length of a given time (in hours).
%           Window is a sliding window around the time in question,
%           except at the beginning and ending of the series, when it is of
%           the length specified but not centered on the time.  Not set to
%           NaN if the measurement at a given time / location is NaN.
%
%  The series length spannned (in time) must be greater than the
%  PerCovWindowSearch
%
%  Input velocities are assumed to be in cm/s
%  Output temporal gradients are in cm/s^2
%
%  Outputs:  TUVOut - the output structure, spatial grid, etc., is the same as the
%  input structure.  The added fields are:
%  TempStats.DuDtFwd:  first entry (UVelTime2-UVelTime1)/TimeDiff, etc., when defined
%            not defined at last time
%  TempStats.DvDtFwd:  (VVelTime2-VVelTime1)/TimeDiff, etc., when defined
%  TempStats.DuDtBwd:  second entry is (UVelTime2-UVelTime1)/TimeDiff, etc., when defined
%            not defined at first time
%  TempStats.DvDtBwd:  second entry is (VVelTime2-VVelTime1)/TimeDiff, etc., when defined
%             not defined at first time
%  TempStats.TotDiffFwd:  sqrt((DuDtFwd^2)+(DvDtFwd^2))
%  TempStats.TotDiffBwd:  sqrt((DuDtBwd^2)+(DvDtBwd^2))
%  TempStats.MaxTotDiff:  maximum of the above two measures
%  TempStats.MinTotDiff:  minimum of the above two measures
%  TempStats.PerCovMin:  Percentage of time that there are measurements at
%  the specified location, for the specified window duration (the minimum
%  percentage for teh sliding windows around the specific measurement time
%  and location)
%  TempStats.PerCovMax:  Percentage of time that there are measurements at
%  the specified location, for the specified window duration (the maximum
%  percentage for the sliding windows around the specific measurement time
%  and location)
%  TempStats.PerCovTyp:  Percentage of time that there are measurements at
%  the specified location, for the specified window duration (the "typical,
%  or mode" percentage for teh sliding windows around the specific measurement 
%  time and location)
%
%  Note:  Putting all of the stats in the subfields of TempStats will make
%  it easier when writing despiking routines later.
%
%  Note:  assumes time differences are in integer units of hours (1 hour, 2
%  hours, or 24 hours, for example).  Not 1.5 hours, etc.  Used to
%  calculate PerCovWithinWindow (empty or nonnexistent measurements are
%  treated as NaN).
%
%
%  Initial step - strip off NaNs in case we have NaNs in the timestamp.  If
%  the NaNs really are at the correct spacing, simply reset the timestamp.
%  The check is to make sure that the new times are unique, and that they
%  are measured at the top of the hour (maybe this check isn't really
%  necessary, but let's leave it in for now).  (Now having this problem with
%  PAFS radials - 20 August 2008).
%
if sum(isnan(TUVIn.TimeStamp))>0,  % means that there are NaNs in the times - will interfere later
    %
    IndicesToKeep=find(~isnan(TUVIn.TimeStamp));
    %
    %
    [NumPoints,NumTimes]=size(TUVIn.U);
    %
    %  radial structues and total structures are interpolated
    %  differently
    %
    %  remove stats fields if we have reached this point, because the
    %  extraction routines do not work on those fields
    %
    if isfield(TUVIn,'TempStats'),
        TUVIn=rmfield(TUVIn,'TempStats');
    end
    if isfield(TUVIn,'SpatialStats'),
        TUVIn=rmfield(TUVIn,'SpatialStats');
    end
    %
    if isequal(upper(TUVIn.Type),'RDLMEASURED')||isequal(upper(TUVIn.Type),'RDLIDEAL'),
        %
        try
            TUVIn=subsrefRADIAL(TUVIn,[1:NumPoints],IndicesToKeep);
        catch
            TUVIn=NaN;
        end
        %
        if ~isstruct(TUVIn),
            disp('GetHFTemporalStats:  Structure supposed to be radial, but extraction routine')
            disp('did not work.  Routine ending, NaN being returned.')
            TUVOut=NaN;
            return;
        end
        %
    else
        %
        try
            TUVIn=subsrefTUV(TUVIn,[1:NumPoints],IndicesToKeep);
        catch
            TUVIn=NaN;
        end
        %
        if ~isstruct(TUVIn),
            disp('GetHFTemporalStats:  Structure supposed to be totals, but extraction routine')
            disp('did not work.  Routine ending, NaN being returned.')
            TUVOut=NaN;
            return;
        end
        %
    end
    %
    %  we now have a valid time axis
    %
end
%
%  Step 1 - Set Up The OutPut Structure.
%
TUVOut=TUVIn;
%
TUVOut.TempStats.DuDtFwd=NaN*ones(size(TUVOut.U));
TUVOut.TempStats.DvDtFwd=NaN*ones(size(TUVOut.U));
TUVOut.TempStats.DuDtBwd=NaN*ones(size(TUVOut.U));
TUVOut.TempStats.DvDtBwd=NaN*ones(size(TUVOut.U));
TUVOut.TempStats.TotDiffFwd=NaN*ones(size(TUVOut.U));
TUVOut.TempStats.TotDiffBwd=NaN*ones(size(TUVOut.U));
TUVOut.TempStats.MaxTotDiff=NaN*ones(size(TUVOut.U));
TUVOut.TempStats.MinTotDiff=NaN*ones(size(TUVOut.U));
TUVOut.TempStats.PerCovMin=NaN*ones(size(TUVOut.U));
TUVOut.TempStats.PerCovMax=NaN*ones(size(TUVOut.U));
TUVOut.TempStats.PerCovTyp=NaN*ones(size(TUVOut.U));
TUVOut.TempStats.PerCovMedian=NaN*ones(size(TUVOut.U));
%
%  Step 3 - At each time, and for each location, get the closest
%  indices both forward in time and backward in time that contain data
%  (U,V) that is not NaN.  Only return the Time Indices if the time difference is
%  less than or equal to MaxGapTDiff.
%
disp('getting closest time indices')
%
IndicesWithMeas=GetClosestTimeIndices(TUVIn,MaxGapTDiff);
%
%  IndicesWithMeas is a structure array with fields FwdIndices and
%  BwdIndices.  The number of indices in each field is equal to the number of
%  lonlatpositions (rows) x number of times (columns).
%
disp('calculating derivatives')
%
TUVOut.TempStats.DuDtFwd=CalcHFTempGradient(TUVIn.U,TUVIn.TimeStamp,IndicesWithMeas.FwdIndices);
TUVOut.TempStats.DvDtFwd=CalcHFTempGradient(TUVIn.V,TUVIn.TimeStamp,IndicesWithMeas.FwdIndices);
TUVOut.TempStats.DuDtBwd=CalcHFTempGradient(TUVIn.U,TUVIn.TimeStamp,IndicesWithMeas.BwdIndices);
TUVOut.TempStats.DvDtBwd=CalcHFTempGradient(TUVIn.V,TUVIn.TimeStamp,IndicesWithMeas.BwdIndices);
%
Tem=(TUVOut.TempStats.DuDtFwd)+(i*TUVOut.TempStats.DvDtFwd);
TUVOut.TempStats.TotDiffFwd=abs(Tem);
%
Tem=(TUVOut.TempStats.DuDtBwd)+(i*TUVOut.TempStats.DvDtBwd);
TUVOut.TempStats.TotDiffBwd=abs(Tem);
%
TUVOut.TempStats.MaxTotDiff=nanmax(TUVOut.TempStats.TotDiffFwd,TUVOut.TempStats.TotDiffBwd);
TUVOut.TempStats.MinTotDiff=nanmin(TUVOut.TempStats.TotDiffFwd,TUVOut.TempStats.TotDiffBwd);
%
%  now calculate the percoverage
%
%
disp('calculating percent coverage windows')
%
PerCovStats=GetPerHFCoverage(TUVIn.U,TUVIn.TimeStamp,PerCovWindowSearch);
%
TUVOut.TempStats.PerCovMin=PerCovStats.MinMeas;
TUVOut.TempStats.PerCovMax=PerCovStats.MaxMeas;
TUVOut.TempStats.PerCovTyp=PerCovStats.TypMeas;
TUVOut.TempStats.PerCovMedian=PerCovStats.MedianMeas;
clear PerCovStats
%
%  !!!!  add calculations here
%
return;
%
function [IndicesOut]=GetClosestTimeIndices(TUVIn,MaxGapTDiff)
%
%  [IndicesOut]=GetClosestTimeIndices(TUVIn,MaxGapTDiff);
%
%  MaxGapTDiff is in hours
%
IndicesOut.FwdIndices=NaN*ones(size(TUVIn.U));
IndicesOut.BwdIndices=NaN*ones(size(TUVIn.U));
%
%  set up a matrix, with the rows equal to the column number (TimeIndex)
%  set up so NaNs if a NaN occurs
%
%  we can at least partially matricize the routine
%
[TotLoc,TotTimes]=size(TUVIn.U);
IndMatrix=ones(TotLoc,1)*[1:TotTimes];
%
temmark=find(isnan(TUVIn.U)|isnan(TUVIn.V));
if numel(temmark)>0,
    IndMatrix(temmark)=NaN;
end
%
%  IndMatrix is now NaN where we have no values
%
for jtime=1:TotTimes,
    %
    switch jtime
        case 1
            FwdMatToCheck=IndMatrix(:,2:TotTimes);
            BackMatToCheck=NaN*ones(TotLoc,1);
        case TotTimes
            FwdMatToCheck=NaN*ones(TotLoc,1);
            BackMatToCheck=IndMatrix(:,1:TotTimes-1);
        otherwise
            BackMatToCheck=IndMatrix(:,1:jtime-1);
            FwdMatToCheck=IndMatrix(:,jtime+1:TotTimes);
    end
    %
    if isequal(TotTimes,1),
        FwdMatToCheck=NaN*ones(TotLoc,1);
        BackMatToCheck=NaN*ones(TotLoc,1);
    end
    %
    [TemRows,TemCols]=size(FwdMatToCheck);
    if TemCols==1,
        FwdMatToCheck=[FwdMatToCheck FwdMatToCheck];
    end
    [TemRows,TemCols]=size(BackMatToCheck);
    if TemCols==1,
        BackMatToCheck=[BackMatToCheck BackMatToCheck];
    end
    %
    FwdMatToCheck=FwdMatToCheck';
    BackMatToCheck=BackMatToCheck';
    %
    TemFwd=nanmin(FwdMatToCheck);
    TemBack=nanmax(BackMatToCheck);
    %
    IndicesOut.FwdIndices(:,jtime)=TemFwd';
    %
    IndicesOut.BwdIndices(:,jtime)=TemBack';
    %
end
%
%  the indices are defined where measurements were found
%  they are NaN otherwise
%
%  We now need to find those places where the time gap is less than 
%  or equal to MaxGapTDiff
%
%  Get a time vector
%
TemTime=TUVIn.TimeStamp;
[mtime,ntime]=size(TemTime);
if mtime>ntime,
    TemTime=TemTime';
end
%
%  Time Matrix associated with each index:
%
TimeIndMatrix=ones(TotLoc,1)*TemTime;
%
%  Time Matrix associated with the forward indices
%  Easiest way is to step through row by row the way it is 
%  set up.  Could matricize by changing the index definitions
%  if speed becomes an issue
%
TimeTemMatrix=NaN*ones(size(TimeIndMatrix));
TimeTemInd=IndicesOut.FwdIndices;
[mtem,ntem]=size(TimeTemMatrix);
for jtem=1:mtem,
    temmark=find(~isnan(TimeTemInd(jtem,:)));
    if numel(temmark)>0,
        TimeTemMatrix(jtem,temmark)=TimeIndMatrix(jtem,TimeTemInd(jtem,temmark));
    end
end
TimeFwdMatrix=TimeTemMatrix;
%
TimeTemMatrix=NaN*ones(size(TimeIndMatrix));
TimeTemInd=IndicesOut.BwdIndices;
[mtem,ntem]=size(TimeTemMatrix);
for jtem=1:mtem,
    temmark=find(~isnan(TimeTemInd(jtem,:)));
    if numel(temmark)>0,
        TimeTemMatrix(jtem,temmark)=TimeIndMatrix(jtem,TimeTemInd(jtem,temmark));
    end
end
TimeBwdMatrix=TimeTemMatrix;
%
TimeFwdDiff=24*(TimeFwdMatrix-TimeIndMatrix);  % now in hours
TimeBwdDiff=24*(TimeIndMatrix-TimeBwdMatrix);
%
temmark=find(TimeFwdDiff>MaxGapTDiff);
if numel(temmark)>0,
    IndicesOut.FwdIndices(temmark)=NaN;
end
%
temmark=find(TimeBwdDiff>MaxGapTDiff);
if numel(temmark)>0,
    IndicesOut.BwdIndices(temmark)=NaN;
end
%
%  Indices are now only returned where the time gap is less than or equal
%  to MaxGapTDiff
%
return;
%
function [GradOut]=CalcHFTempGradient(FieldIn,TimesIn,IndicesIn)
%
%  calculates the gradients
%  FieldIn is (typically) either U or V in cm/s.  rows are locations,
%  columns are different times
%  TimesIn is the time vector in days (converted to seconds for gradient
%  calculation)
%  IndicesIn is the time index of the measuremnt to use when calculating
%  the gradient.  If time indices in each column are greater than the
%  column number, a forward derivative is calculated.  If less, then a
%  bacckward derivative is calculated (gradient alwayss defined as the
%  measurement at teh largest time minus that at the smallest time)
%
%  GradientOut is the same size as FieldIn
%
[mtime,ntime]=size(TimesIn);
if mtime>ntime,
    TimesIn=TimesIn';
end
%
[TotLoc,TotTimes]=size(FieldIn);
TemTimeMatrix=ones(TotLoc,1)*TimesIn;
TemIndMatrix=ones(TotLoc,1)*[1:TotTimes];
%
%  set up the measurement matrix to use when calculating the gradients
%  also find time associated with each measurement
%
EndPointMatrix=NaN*ones(size(FieldIn));
EndTimeMatrix=NaN*ones(size(FieldIn));
%
TimeTemInd=IndicesIn;
[mtem,ntem]=size(EndPointMatrix);
for jtem=1:mtem,
    temmark=find(~isnan(TimeTemInd(jtem,:)));
    if numel(temmark)>0,
        EndPointMatrix(jtem,temmark)=FieldIn(jtem,TimeTemInd(jtem,temmark));
        EndTimeMatrix(jtem,temmark)=TemTimeMatrix(jtem,TimeTemInd(jtem,temmark));
    end
end
%
%  can use either mode or nanmedian to decide if the indices we are looking
%  at are forward or backward looking
%  both cases mathematically are the same, however am making expliciit here
%  to allow for an error check (the case where no time difference is
%  accidentally specified).
%
IndexDiff=IndicesIn-TemIndMatrix;
CheckGradDirection=mode(reshape(IndexDiff,TotLoc*TotTimes,1));
switch sign(CheckGradDirection)
    case 1
        MeasDiff=EndPointMatrix-FieldIn;
        TimeDiff=24*(EndTimeMatrix-TemTimeMatrix); % now in hours
        TimeDiff=3600*TimeDiff;  % now in seconds
    case -1
        MeasDiff=FieldIn-EndPointMatrix;
        TimeDiff=24*(TemTimeMatrix-EndTimeMatrix);
        TimeDiff=3600*TimeDiff;  % now in seconds
    otherwise
        disp('time indices for gradient incorrect, or no valid measurements')
        disp('found.  returning NaNs in the gradient')
        MeasDiff=NaN*ones(size(FieldIn));
        TimeDiff=NaN*ones(size(FieldIn));
end
%
GradOut=MeasDiff./TimeDiff;
%
return;
%
function [PerCovStats]=GetPerHFCoverage(FieldIn,MeasDNumTimes,WindowLength)
%
%  GetPerHFCoverage(FieldIn,MeasDNumTimes,WindowLength);
%
%  FieldIn are teh measurements, rows are positions, columns are times
%  MeasDNumTimes is the associated time axis measured in days
%  WindowLength is the length of the sliding window to use in hours
%
%  PerCovStats.MinMeas - the mimum percent coverage found for the sliding
%  window at each point and time
%  PerCovStats.MaxMeas - the maximum percent coverage found for the sliding
%  window at each point and time
%  PerCovStats.TypMeas - the typical percent coverage found for the sliding
%  window at each point and time
%
%  Modified to assume hourly measurements!  Nov 2008
%
PerCovStats.MinMeas=NaN*ones(size(FieldIn));
PerCovStats.MaxMeas=NaN*ones(size(FieldIn));
PerCovStats.TypMeas=NaN*ones(size(FieldIn));
PerCovStats.MedianMeas=NaN*ones(size(FieldIn));
%
[TotPoints,TotTimes]=size(FieldIn);
%TemTimeDiff=MeasDNumTimes(2:TotTimes)-MeasDNumTimes(1:TotTimes-1);
%TemTimeDiff=24*TemTimeDiff;  % now in hours
%TypTimeDiff=mode(TemTimeDiff);  % typical spacing in hours
%
MaxMeasPerWindow=WindowLength+1;  % b/c of the way that the window length
%  is used later in the routine
MaxWindowsPerMeasurement=MaxMeasPerWindow;  %  the number of sliding windows
%
FieldTimeStart=nanmin(MeasDNumTimes);
FieldTimeEnd=nanmax(MeasDNumTimes);
%
for jtime=1:TotTimes,
    %
    if isequal(rem(jtime,50),0),
        TemStr=['Calculating percent cov windows, time ' num2str(jtime) ' of ' num2str(TotTimes)];
        disp(TemStr)
    end
    %
    TemCovMeas=NaN*ones(TotPoints,MaxWindowsPerMeasurement);
    %
    TimeToGet=MeasDNumTimes(jtime);
    %
    for jwind=1:MaxWindowsPerMeasurement,
        %
        TimeStart=TimeToGet-((MaxMeasPerWindow-1)/24)-0.001+((jwind-1)/24);
        TimeEnd=TimeStart+((MaxMeasPerWindow-1)/24)+0.002;
        %
        if (TimeStart>=FieldTimeStart)&&(TimeEnd<=FieldTimeEnd),
            %
            IndicesToObtain=find((MeasDNumTimes>=TimeStart)&(MeasDNumTimes<=TimeEnd));
            %
            if numel(IndicesToObtain>0),
                %
                FieldExtract=FieldIn(:,IndicesToObtain)';
                FieldExtract=sum(~isnan(FieldExtract));
                TemCovMeas(:,jwind)=FieldExtract;
                %
            end
        end
    end
    %
    temmark=find(~isnan(nansum(TemCovMeas)));
    if numel(temmark)>0,
        TemCovMeas=TemCovMeas(:,temmark);
        TemSer=nanmin(TemCovMeas');
        PerCovStats.MinMeas(:,jtime)=TemSer';
        TemSer=nanmax(TemCovMeas');
        PerCovStats.MaxMeas(:,jtime)=TemSer';
        TemSer=mode(TemCovMeas');
        PerCovStats.TypMeas(:,jtime)=TemSer';
        TemSer=nanmedian(TemCovMeas');
        PerCovStats.MedianMeas(:,jtime)=TemSer';
    end
    %
end
%
PerCovStats.MinMeas=(100/MaxMeasPerWindow)*PerCovStats.MinMeas;
PerCovStats.MaxMeas=(100/MaxMeasPerWindow)*PerCovStats.MaxMeas;
PerCovStats.TypMeas=(100/MaxMeasPerWindow)*PerCovStats.TypMeas;
PerCovStats.MedianMeas=(100/MaxMeasPerWindow)*PerCovStats.MedianMeas;
%
%  the routine is done
%
return;
%
