function Tout = averageHFTotals( Tin,AvInterval,PercentConsideredGood,SpatialSwitch,TemporalSwitch)
%
%  Tout = averageHFTotals( Tin,AvInterval,PercentConsideredGood,SpatialSwitch,TemporalSwitch);
%
%  This function averages the totals data (for example, uses hourly totals
%  to get daily totals).  the resulting totals are spatially and temporally
%  interpolated if the appropriate switches at the end are thrown.
%
%  The input time axis should be increasing, but does not have to have
%  uniform spacing.
%
%  If both spatial and temporal switches are thrown, spatial interpolation
%  will be performed first.
%
%  Note:  the way the routine is set up currently, an odd number for the
%  AvInterval means that the correct averaging interval is returned. An even number
%  means that each average is actually an AvInterval+1 average (so, 24
%  hours = a 25 hour average)
%
%  Required Inputs:
%  Tin = the TUV structure of totals
%  AvInterval = the averaging interval in hours.  Outputs will be centered
%  (so for example, daily averages will be centered at noon GMT).
%  PercentConsideredGood = the percentage of valid non-NaN measurements at
%  a given location that need to be present in order to return a valid
%  "average".  Somewhere around 40 is probably good.
%
%  Optional inputs:
%  SpatialSwitch - if zero or NaN (or not included in the call), no spatial
%  interpolation is done.  Otherwise performed using the spatial interp
%  routine, which grids the totals and extracts them when it can.
%  TemporalSwitch - if zero or NaN (or not included in the call), no
%  temporal interpolation is done.  Otherwise it temporally interpolates
%  over "TemporalSwitch" averaged values.  In other words, if obtaining
%  daily averages, a value of TemporalSwitch=3 means to interpolate over 3
%  day gaps.
%
%  Outputs:  Tout = The TUV structure.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% 	$Id: averageHFTotals $	
%
% Copyright (C) 2008 Chris Halle
% Licence: GPL
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%  Step 1.  Set up the output time vector
%
TimeStart=datevec(Tin.TimeStamp(1));
TimeStart=datenum([TimeStart(1) TimeStart(2) TimeStart(3) 0 0 0]);
TimeStart=TimeStart+((1/24)*(AvInterval/2));
MaxTime=Tin.TimeStamp(numel(Tin.TimeStamp));
TimeAxisOut=TimeStart:(AvInterval/24):MaxTime;
clear TimeStart MaxTime
%
%  Step 2 - How many measurements should be at each location in order to be
%  considered valid?
%
tdiff=Tin.TimeStamp(2:numel(Tin.TimeStamp))-Tin.TimeStamp(1:numel(Tin.TimeStamp)-1);
tdiff=tdiff*24;  %  time difference now in hours
TypTimeDiff=mode(tdiff);  % keep TypTimeDiff to use later - account for roundoff, etc.
%  will use when searching for matching times
%
NumTypMeas=AvInterval/TypTimeDiff;
NumMeasNeeded=(PercentConsideredGood/100)*NumTypMeas;
clear NumTypMeas tdiff
%
%  Step 3 - Set up output structure, loop through, get measurements
%
[NumLocations,NumCols]=size(Tin.LonLat);
Tout = TUVstruct([NumLocations numel(TimeAxisOut)],1);
Tout.TimeStamp=TimeAxisOut;
Tout.U(:,:)=NaN;
Tout.V(:,:)=NaN;
Tout.LonLat=Tin.LonLat;
%
for jtime=1:numel(TimeAxisOut),
    temmark=find((Tin.TimeStamp>=TimeAxisOut(jtime)-(AvInterval/24)-(0.01*TypTimeDiff))&...
        (Tin.TimeStamp<=TimeAxisOut(jtime)+(AvInterval/24)+(0.01*TypTimeDiff)));  %TypTimeDiff used to get endpoints
    if numel(temmark)>0,
        TemU=Tin.U(:,temmark);
        TemV=Tin.V(:,temmark);
        [mrows,ncols]=size(TemU);
        if ncols>1,
            TemU=TemU';
            MeanU=nanmean(TemU);
            SetMeanU=NaN*ones(size(MeanU));
            TemV=TemV';
            MeanV=nanmean(TemV);
            SetMeanV=NaN*ones(size(MeanV));
            NumMeas=sum(~isnan(TemU+TemV));
            TemMeas=find(NumMeas>=NumMeasNeeded);
            if numel(TemMeas)>0,
                SetMeanU(TemMeas)=MeanU(TemMeas);
                SetMeanV(TemMeas)=MeanV(TemMeas);
                Tout.U(:,jtime)=SetMeanU';
                Tout.V(:,jtime)=SetMeanV';
            end
        else
            if abs((AvInterval/24)-TypTimeDiff)<=(0.01*TypTimeDiff), % means that we do not want to average
                Tout.U(:,jtime)=Tin.U(:,temmark);
                Tout.V(:,jtime)=Tin.V(:,temmark);
            end
        end
    end
end
%
%  We now have the averaged data
%
%  Step 4 - spatial interpolation
%
%  spatial interpolation has a bug - leave off for now
%
if exist('SpatialSwitch','var'),
    if ~isequal(SpatialSwitch,0)&&~isnan(SpatialSwitch),
        Tout = spatialInterpTotals(Tout);
    end
end
%
%  Step 5 - temporal interpolation
%
if exist('TemporalSwitch','var'),
    if ~isequal(TemporalSwitch,0)&&~isnan(TemporalSwitch),
        MaxGapInterp=(AvInterval*TemporalSwitch)/24;  % max gap allowed in days.  Interface with the interp routine o.k.?        
        Tout=temporalInterpTotals(Tout,Tout.TimeStamp,MaxGapInterp);
    end
end
%
%  the routine is done
%
return;
%
