function [R,range,bear,head] = interpRadialsRorig2( R, varargin )
% interpRadialsRorig2  Used for range and bear interpolation of radials.  
%
%  modification of DMK code interpRadials b/c it didn't work for higher
%  resolution.  Can be used for to interpolate to higher or lower
%  resolution, but be careful.  This is interpolation, not averaging.
%
%  If there are more than 2 measurements near a given "new" location (within a distance
%  of MaxRangeGap), and the angles from the new to the old locations cover
%  a range of angles greater than 90 degrees, then the new measurement is
%  kept.
%
%  If a new location is within "GridSlop" of an old location, it is kept.
%  The new value is an interpolated value, however.
%
%  BearLims for the input of this routine is measured CCW from east (math convention).
%
%  Interpolation ("Wraparound") of angles is handled by reflecting old measurements about the
%  360 degree line (so angles from 340 to 360 are reflected about 360).
%  Angles from 0 to 20 are reflected the other way.
%
%  CMH, March 2009
%
% Usage: [RADIAL,RANGE,BEAR,HEAD] = interpRadials( RADIAL, PARAM1, VAL1, ..., PARAMn, VALn )
%
% Inputs
% ------
% RADIAL = A radial structure to be interpolated.  This must be a single
% radial structure, as opposed to an array of radial structures with
% multiple sites.
%
%  The parameters should be specified.  CMH
%
% PARAMn,VALn = string parameter names and associated values.  The list
% of possible parameters is:
%
% RangeLims: Can be either a three element vector of
%            [ rangelowerlim, rangeincrement, rangeupperlim ] or a
%            vector of range values.  If absent or empty, defaults to
%            unique(RADIAL.RangeBearHead(:,1)).  This could cause problems
%            if 3 ranges are in RADIAL structure.
% BearLims: Can be either a three element vector of [ bearlowerlim,
%            bearincrement, bearupperlim ] or a vector of bear values.
%            If absent or empty, defaults to unique(RADIAL.RangeBearHead(:,2)).
%            This could cause problems if only three bears are in RADIAL
%            structure. Note that if a three element vector is given, then
%            angleVector will be used to generate the list of bear values.
% MaxRangeGap: size of the largest range gap in array index units over which
%              interpolation is to be performed.  Defaults to 2.5 (results
%              in filling in single gaps in range).  Set to zero to turn off
%              range interpolation (though data will still be placed on the
%              grid).
%
% Outputs
% -------
% RADIAL = RADIAL structure after interpolation.  Interpolation will only be
% performed over U, V, RadComp and Error.  The Flag will contain a number
% indicating where the data came from: 1 = original data, 2 = range
% interpolation, 4 = bear interpolation, 6 = average of both.
%
% RANGE = Range grid on which the interpolation was performed.
% BEAR = Bear grid on which the bear interpolation was performed.
% HEAD = Heading grid corresponding to RANGE, HEAD and SiteOrigin.
% 
%
% NOTE: No attempt will be made to interpolate the variables in
% RADIAL.OtherMatrixVars.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% 	$Id: interpRadials.m 396 2007-04-02 16:56:29Z mcook $	
%
% Copyright (C) 2007 David M. Kaplan
% Licence: GPL (Gnu Public License)
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% First deal with parameters passed to function.
% Process and validate.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
OldLonLat=R.LonLat;
%
ThetaConv=pi/180;

%
%defaults

p.RangeLims=3.5;
p.BearLims=2.5;
p.MaxRangeGap=10;
p.GridSlop=1;

% known parameters
param_list = [ fieldnames(p)', {'RangeLims','BearLims','MaxRangeGap','GridSlop'} ];

[p,pb] = checkParamValInputArgs( p, param_list, {}, varargin{:} );


% Warn if found some unknown parameters.
if ~isempty(pb)
  pb = strcat(fieldnames(pb), {' '} );
  pb = [ pb{:} ];
  warning( ['The following unknown parameters were given and will be ' ...
            'ignored: %s '], pb );
  clear pb
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Now create grids if necessary.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if ~isfield( p, 'RangeLims' ) || isempty(p.RangeLims)
  range = unique(R.RangeBearHead(:,1));
  p.RangeLims = 'VECTOR';
elseif numel(p.RangeLims) == 3
  range = p.RangeLims(1):p.RangeLims(2):p.RangeLims(3);
else
  range = p.RangeLims;
  p.RangeLims = 'VECTOR';
end

if ~isfield( p, 'BearLims' ) || isempty(p.BearLims)
  bear = unique(R.RangeBearHead(:,2));
  p.BearLims = 'VECTOR';
elseif numel(p.BearLims) == 3
  bear = angleVector( p.BearLims(1), p.BearLims(2), p.BearLims(3) );
else
  bear = p.BearLims;
  p.BearLims = 'VECTOR';
end

if ~isfield( p, 'GridSlop' ) || isempty(p.GridSlop)
  GridSlop=1;
else
    GridSlop=p.GridSlop;
end

if ~isfield( p, 'MaxRangeGap' ) || isempty(p.MaxRangeGap)
  MaxRangeGap=10;
else
    MaxRangeGap=p.MaxRangeGap;
end


disp('creating grids and interpolation')
pause(1);

% Create grids
[range,bear] = meshgrid( range, bear );

% Generate equivalent lon, lat and headings using m_fdist
[Lon,Lat,head] = m_fdist( R.SiteOrigin(1), R.SiteOrigin(2), math2true(bear), ...
                          range * 1e3 );

% Fix longitude so that it is -180 to 180
Lon(Lon>180) = Lon(Lon>180) - 360;

R.LonLat=[reshape(Lon,numel(Lon),1) reshape(Lat,numel(Lat),1)];

%  Get ranges and bearings for interpolation

[sold,a12old,a21]=m_idist(R.SiteOrigin(1),R.SiteOrigin(2),OldLonLat(:,1),OldLonLat(:,2),'wgs84');

[snew,a12new,a21]=m_idist(R.SiteOrigin(1),R.SiteOrigin(2),R.LonLat(:,1),R.LonLat(:,2),'wgs84');

RadCompOld=R.RadComp;
ErrOld=R.Error;

% deal with wraparound (use +/- 20 degrees or so)...  onnly works if we
%  have continuous 0-360 measurements, otherwise makes no difference
AngleCut=20;
temmark=find(a12old<=AngleCut);
if numel(temmark)>0,
    NewAngles1=a12old(temmark)+360;
    NewS1=sold(temmark);
    NewRad1=RadCompOld(temmark);
    NewErr1=ErrOld(temmark);
else
    NewAngles1=[];
    NewS1=[];
    NewRad1=[];
    NewErr1=[];
end
temmark=find(a12old>=(360-AngleCut));
if numel(temmark)>0,
    NewAngles2=a12old(temmark)-360;
    NewS2=sold(temmark);
    NewRad2=RadCompOld(temmark);
    NewErr2=ErrOld(temmark);
else
    NewAngles2=[];
    NewS2=[];
    NewRad2=[];
    NewErr2=[];
end
%
a12old=[a12old;NewAngles1;NewAngles2];
sold=[sold;NewS1;NewS2];
RadCompOld=[RadCompOld;NewRad1;NewRad2];
ErrOld=[ErrOld;NewErr1;NewErr2];
%

RadCompNew=griddata(sold/1000,a12old,RadCompOld,snew/1000,a12new);
ErrNew=griddata(sold/1000,a12old,ErrOld,snew/1000,a12new);

%  now find those that are valid

disp('calculating distances to old grids')
pause(1);

%
NumOldLon=numel(OldLonLat(:,1));
NumNewLon=numel(R.LonLat(:,1));

[s,a12,a21] = m_idist(R.LonLat(:,1)*ones(1,NumOldLon),R.LonLat(:,2)*ones(1,NumOldLon),ones(NumNewLon,1)*(OldLonLat(:,1)'),ones(NumNewLon,1)*(OldLonLat(:,2)'),'wgs84');

%  NaNs will occur if points are too close

temmark=find(isnan(s));
if numel(temmark)>0,
    s(temmark)=0;
    a12(temmark)=0;
    a21(temmark)=0;
end
%

s=s/1000;

KeepPointsGridSlop=sign(sum(s'<GridSlop));
KeepPointsGridSlop=KeepPointsGridSlop';

s(s>=MaxRangeGap)=NaN;

temmark=find(isnan(s));
if numel(temmark)>0,
    a12(temmark)=NaN;
end


NumPointsNearby=sum(~isnan(s'));
NumPointsNearby=NumPointsNearby';

%  if dot product of any angle with others is negative, then we have a
%  spread of greater than 90 degrees.  Loop is slow, but will save memory

AngleDiff=NaN*ones(numel(NumPointsNearby),1);
for jangle=1:numel(AngleDiff),
    temmark=find(~isnan(a12(jangle,:)));
    if numel(temmark)>1,
        if isequal(rem(jangle,100),0),
            TemStr=['Grid : ' num2str(jangle) ' of ' num2str(numel(AngleDiff)) ' being checked'];
            disp(TemStr);
        end
        [AngleDiff(jangle),anglePair] = angSpread2(a12(jangle,temmark));
    end
end

AngleDiff(AngleDiff<90)=NaN;


IndicesGridSlop=find(KeepPointsGridSlop>0.5);
IndicesOtherPoints=find((NumPointsNearby>=2)&(~isnan(AngleDiff)));

IndicesAll=union(IndicesGridSlop,IndicesOtherPoints);


R.LonLat=R.LonLat(IndicesAll,:);
R.RadComp=RadCompNew(IndicesAll);
R.Error=ErrNew(IndicesAll);

clear RadCompOld OldLonLat RadComp

[range,bear,head]=m_idist(R.SiteOrigin(1),R.SiteOrigin(2),R.LonLat(:,1),...
    R.LonLat(:,2),'wgs84');

head = true2math( head );
bear = true2math( bear );

range=range/1000;

R.RangeBearHead=[reshape(range,numel(range),1) reshape(bear,numel(bear),1) reshape(head,numel(head),1)];


R.U=R.RadComp.*cos(ThetaConv*reshape(head,numel(head),1));
R.V=R.RadComp.*sin(ThetaConv*reshape(head,numel(head),1));;
R.Flag=isfinite(R.RadComp);



R.ProcessingSteps{end+1} = mfilename;

% Add metadata
R.OtherMetadata.(mfilename) = p;


%
return;
%
